package examples.prototype.java;

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the design patterns project at UBC
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ca.ubc.cs.spl.pattern.
 *
 * Contributor(s):   
 */

/**
 * Implements the driver for the Prototype design pattern example.<p> 
 *
 * Intent: <i>Specify the kinds of objects to create using a prototypical 
 * instance, and create new objects by copying this prototype.</i><p>
 *
 * Participatng objects are <code>MyString</code> and 
 * <code>AnotherString</code> as <i>Prototype</i>s.<p>
 *
 * In this example, both MyString and AnotherString implement cloneable
 * classes emulating limited String behavior. This driver creates an 
 * object of each class and clones it. Both originals and clones are
 * manipulated to show that they are different objects. 
 *
 * <p><i>This is the Java version.</i><p> 
 *
 * Java's <code>Cloneable</code> interface is used.
 *
 * @author  Jan Hannemann
 * @author  Gregor Kiczales
 * @version 1.0, 05/13/02
 * 
 * @see MyString
 * @see AnotherString
 */ 
 
public class Main {

    /**
     * Implements the driver for the Prototype design pattern example.<p> 
     *
     * In this example, both MyString and AnotherString implement cloneable
     * classes emulating limited String behavior. This driver creates an 
     * object of each class and clones it. Both originals and clones are
     * manipulated to show that they are different objects. 
     *
     * @param args the command line parameters, unused.
     */ 

    public static void main(String[] args) {
        
        try {
    
            System.out.println("Testing the Prototype design pattern implementation...");
            
            MyString      orig1;
            AnotherString orig2;
            MyString      copy1, copy3;
            AnotherString copy2;
            
            orig1 = new MyString("  This is Prototype 1");
            orig2 = new AnotherString("  This is Prototype 2"); 
            
            System.out.println("These are the two prototypes:");
            System.out.println(orig1);
            System.out.println(orig2);
            
            copy1 = (MyString) orig1.clone();
            copy2 = (AnotherString) orig2.clone();
    
            System.out.println("These are copies of the prototypes:");
            System.out.println(copy1);
            System.out.println(copy2);
    
            orig1.setText("  This is Prototype 3");
            
            System.out.println("Now prototype 1 was changed. Here are p1 and its former copy:");
            System.out.println(orig1);
            System.out.println(copy1);
            
            copy3 = (MyString) orig1.clone();
            copy2.setText("  This is Clone 2");
                    
            System.out.println("This is a clone of the changed p1 and a changed copy2:");
            System.out.println(copy3);
            System.out.println(copy2);
            
            
            System.out.println("... done.");  
        
        } catch (CloneNotSupportedException ex) {
            System.err.println("Failure! "+ex);
        }

    }
}